import json
import argparse
import os.path
from paffi import get_nodes_keys, get_node_latitude, get_node_longitude, get_delays, get_delay_destination, \
    get_delay_source

parser = argparse.ArgumentParser()
parser.add_argument("-i", "--input", help=".json input file for topology. Default = conn_topology.json")
parser.add_argument("-t", "--template", help="gnuplot template prefix. Default = gnuplot_resources/map_template")
parser.add_argument("-o", "--output", help="Prefix for output file [.dat, .gnuplot]. Default = <input_filename>")
args = parser.parse_args()

infile = args.input if args.input else "conn_topology.json"
gnuplottemplate = "%s.gnuplot" % args.template if args.template else "gnuplot_resources/map_template.gnuplot"

if args.output:
    dataout = "%s.dat" % args.output
    gnuplotout = "%s.gnuplot" % args.output
    epsout = "%s.eps" % args.output
    pngout = "%s.png" % args.output
else:
    dataout = "%s.dat" % os.path.splitext(infile)[0]
    gnuplotout = "%s.gnuplot" % os.path.splitext(infile)[0]
    epsout = "%s.eps" % os.path.splitext(infile)[0]
    pngout = "%s.png" % os.path.splitext(infile)[0]


with open(infile) as LD:
    topology = json.load(LD)

cloudlines = '"<(grep -P \'^3\\t\' %s)" using ($5):($4) notitle with p pt 15 lc -1 ps 3' % dataout
foglines = ''
sensorlines = ''
for i in range(len(get_nodes_keys(topology, 'fog'))):
    foglines += '"<(grep -P \'^2\\t%i\\t\' %s)" using ($5):($4) notitle with p pt %i lc %i ps 2, \\\n' % \
                (i, dataout, (i * 2) % 10 + 5, i+1)
    sensorlines += '"<(grep -P \'^1\\t[0-9]+\\t%i\' %s)" using ($5):($4) notitle with p pt %i lc %i ps 2, \\\n' % \
                   (i, dataout, (i * 2) % 10 + 4, i+1)

to_replace = {"*SENSORS*": sensorlines,
              "*FOGNODES*": foglines,
              "*CLOUDS*": cloudlines,
              "*EPSFILENAME*": epsout,
              "*PNGFILENAME*": pngout
              }

connsf = {}
for i in get_delays(topology, delaytype='conn_sf'):
    connsf[get_delay_source(topology, delaytype='conn_sf', lbl=i)] = get_delay_destination(topology,
                                                                                           delaytype='conn_sf', lbl=i)
connfc = {}
for i in get_delays(topology, delaytype='conn_fc'):
    connfc[get_delay_source(topology, delaytype='conn_fc', lbl=i)] = get_delay_destination(topology,
                                                                                           delaytype='conn_fc', lbl=i)
# Create map
# map is in format:
# type, id, group_id, lat, lon
# type = [1 = sensor, 2 = fog, 3 = cloud]
with open(dataout, "w") as fout:
    for i in get_nodes_keys(topology, 'sensors'):
        fout.write("%i\t%s\t%s\t%s\t%s\n" %
                   (1, i, connsf[i], get_node_latitude(topology, 'sensors', i),
                    get_node_longitude(topology, 'sensors', i)))

    for i in get_nodes_keys(topology, 'fog'):
        fout.write("%i\t%s\t%s\t%s\t%s\n" %
                   (2, i, connfc[i], get_node_latitude(topology, 'fog', i), get_node_longitude(topology, 'fog', i)))

    for i in get_nodes_keys(topology, 'cloud'):
        fout.write("%i\t%s\t%s\t%s\t%s\n" %
                   (3, i, '-1',
                    get_node_latitude(topology, 'cloud', i), get_node_longitude(topology, 'cloud', i)))

with open(gnuplottemplate, 'r') as tfile:
    template = tfile.readlines()

with open(gnuplotout, "w") as fout:
    for line in template:
        for k in to_replace:
            line = line.replace(k, to_replace[k])
        fout.write(line)
