import json
import argparse
from paffi import get_avg_delay, scale_delay, get_nodes_keys, set_node_attr, save_topology, load_topology

parser = argparse.ArgumentParser()
parser.add_argument("-i", "--input", help=".json input file for topology. Default = conn_topology.json")
parser.add_argument("-d", "--delta", help="average network delay. Default = 10e-3")
parser.add_argument("-m", "--deltamu", help="delta * mu parameter. Default = 1")
parser.add_argument("-r", "--rho", help="rho parameter. Default = 0.5")
parser.add_argument("-o", "--output", help="Name for output file. Default = scenario.json")
args = parser.parse_args()

infile = args.input if args.input else "conn_topology.json"
out = args.output if args.output else "scenario.json"

with open(infile) as LD:
    topology = load_topology(infile)

delta = float(args.delta) if args.delta else 10e-3
# normalize_delay(topology, ['delay_sf', 'delay_fc'], delta)
d = get_avg_delay(topology, ['delay_sf', 'delay_fc'])
scale_delay(topology, ['delay_sf', 'delay_fc', 'conn_sf', 'conn_fc', 'conn_ff'], delta / d)

delta_mu = float(args.deltamu) if args.deltamu else 1.0
rho = float(args.rho) if args.rho else 0.5

n_src = len(get_nodes_keys(topology, 'sensors'))
n_fog = len(get_nodes_keys(topology, 'fog'))
n_cloud = len(get_nodes_keys(topology, 'cloud'))

# delta is the average delay: we set it to 10ms
mu_fog = delta_mu / delta
lambda_src = rho * mu_fog * (n_fog / n_src)

for i in get_nodes_keys(topology, 'sensors'):
    set_node_attr(topology, 'sensors', i, 'lambda', str(lambda_src))

for i in get_nodes_keys(topology, 'fog'):
    set_node_attr(topology, 'fog', i, 'mu', str(mu_fog))

# print(lambda_src, mu_fog)
save_topology(topology, out)
