import argparse
import random
from paffi import load_topology, get_nodes_keys, create_topology, save_topology, set_node, \
    get_node_longitude, get_node_latitude, get_node_attr, compute_distances

parser = argparse.ArgumentParser()
parser.add_argument("-s", "--sensorlist", help="File containing list of sensors. Default: sensors.json")
parser.add_argument("-f", "--foglist", help="File containing list of fog nodes. Default: fog.json")
parser.add_argument("-c", "--cloudlist", help="File containing list of cloud data centers. Default: cloud.json")
parser.add_argument("-S", "--sensornum", help="number of sensors. Default: all")
parser.add_argument("-F", "--fognum", help="number of fog nodes. Default: all")
parser.add_argument("-C", "--cloudnum", help="number of cloud data centers. Default: all")
parser.add_argument("-o", "--output", help="Choose a name for your output file. Default = topology.json")
args = parser.parse_args()


def add_subset(topo_in, topo_out, nodetype, n):
    nin = len(get_nodes_keys(topo_in, nodetype))
    if n < 0 or n > nin:
        n = nin
    indexes = list(range(0, nin))
    random.shuffle(indexes)
    for i in range(0, n):
        set_node(topo_out, nodetype, i,
                 lat=get_node_latitude(topo_in, nodetype, str(indexes[i])),
                 lon=get_node_longitude(topo_in, nodetype, str(indexes[i])),
                 address=get_node_attr(topo_in, nodetype, str(indexes[i]), 'address'))


fsens = args.sensorlist if args.sensorlist else "sensors.json"
ffog = args.foglist if args.foglist else "fog.json"
fcloud = args.cloudlist if args.cloudlist else "cloud.json"
fout = args.output if args.output else "topology.json"
nsens = int(args.sensornum) if args.sensornum else -1
nfog = int(args.fognum) if args.fognum else -1
ncloud = int(args.cloudnum) if args.cloudnum else -1

sens = load_topology(fsens)
fog = load_topology(ffog)
cloud = load_topology(fcloud)

topology = create_topology()
add_subset(sens, topology, 'sensors', nsens)
add_subset(fog, topology, 'fog', nfog)
add_subset(cloud, topology, 'cloud', ncloud)
compute_distances(topology, 'sensors', 'fog')
compute_distances(topology, 'fog', 'cloud')
compute_distances(topology, 'fog', 'fog')

save_topology(topology, fout)
